package utilities;

import java.net.DatagramPacket;
import java.nio.ByteBuffer;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

///example HexUtilities.hexToHexString(byte[] dataByte,' ')

public class HexUtilities
{
	protected static transient final Log logger = LogFactory.getLog(HexUtilities.class);

	private HexUtilities()
	{
	}
	
	public static void print()
	{
	}
	
	public static void print(boolean more)
	{
	}
	
	public static void print(byte[] data)
	{
		logger.info("Data: " + hexToHexString(data));
	}

	public static void print(ByteBuffer data)
	{
		logger.info("Data: " + hexToHexString(data.array(),data.array().length,' '));
	}

	public static void print(DatagramPacket packet)
	{	//print out the data byte received inside the packet
		logger.info("Data: " + hexToHexString(packet.getData(),0,packet.getLength(),' '));
	}

	public static String print2Str(DatagramPacket packet)
	{	//print out the data byte received inside the packet
		return(hexToHexString(packet.getData(),0,packet.getLength(),' '));
	}

	public static String hexToHexString(byte hex)
	{	//convert hex=0xAB, result -> String="AB"
		char upper, lower;
		upper = (char) ((hex >> 4) & 0x0F);
		lower = (char) (hex & 0x0F);
		return(Integer.toHexString(upper).toUpperCase() + Integer.toHexString(lower).toUpperCase());
	}

	public static String hexToHexString(byte[] dataByte)
	{
		return(hexToHexString(dataByte,0,dataByte.length,(char)0));
	}
	
	public static String hexToHexString(byte[] dataByte, int length)
	{
		return(hexToHexString(dataByte,0,length,(char)0));
	}

	public static String hexToHexString(byte[] dataByte, char charBetween)
	{
		return(hexToHexString(dataByte,0,dataByte.length,charBetween));
		
	}

	public static String hexToHexString(byte[] dataByte, int length, char charBetween)
	{
		return(hexToHexString(dataByte,0,length,charBetween));
		
	}
	public static String hexToHexString(byte[] dataByte, int startIndex, int length, char charBetween)
	{	//dataByte=0x01 0x02 0x03 0x04 0x05 0x06 0x07 0x08, startIndex=2, length=3, charBetween=':' 
		//result -> String="03:04:05"
		//dataByte=0x01 0x02 0x03 0x04 0x05 0x06 0x07 0x08, startIndex=2, length=3, charBetween='0' 
		//result -> String="030405"
		StringBuffer buffer = new StringBuffer();
		int endIndex=startIndex+length;				//compute the index to stop the iteration
		for (int x = startIndex; x < endIndex; x++)
		{
			buffer.append(hexToHexString(dataByte[x]));
			if((charBetween!=0x00)&&(x < endIndex-1))	//if char is not a null. End of string is not terminated with the charBetween
				buffer.append(charBetween);		//insert white space in between the hex ascii		
		}
		return buffer.toString();
	}
	
	//IP utilities
	public static byte[] ipStr2Hex(String ip)
	{	
		//ip=192.168.234.20
		//result byte y[]={0xC0, 0xA8, 0xEA, 0x14 }
		byte y[]={0,0,0,0};
		
		String ipInt[];
		ipInt=ip.split("\\.",4);			//need to use \\ in order to use the dot

		for(int x=0;x<4;x++)
		{
			ipInt[x]=ipInt[x].trim();		//trim possible white space in the string
			y[x] = (byte)Integer.parseInt(ipInt[x]); 	// convert from String to int to byte
		}
		return(y);
	}
	
	public static byte[] int2Byte(int num)
	{
		byte y[] = {0,0};
		y[0]=(byte)(num&0x00FF);
		y[1]=(byte)((num>>8)&0x00FF);
		return(y);
	}

	public static byte[] int2ByteReversed(int num)
	{
		byte y[] = {0,0};
		y[1]=(byte)(num&0x00FF);
		y[0]=(byte)((num>>8)&0x00FF);
		return(y);
	}

	public static long byte2Long(byte byte4[])
	{	//convert a 4 byte into a long type
		long data=0;
		if(byte4.length!=4)
			logger.error("byte2Long() -> wrong byte length");
		
		data|=(byte4[0]<<24)&0xFF000000;
		data|=(byte4[1]<<16)&0x00FF0000;
		data|=(byte4[2]<<8)&0x0000FF00;
		data|=(byte4[0])&0x000000FF;
		return(data);
	}

	public static int compare(byte[] a, byte[] b)
	{
		return(compare(a, b, 0, 0));
	}

	public static int compare(byte[] a, byte[] b, int b_offset)
	{
		return(compare(a, b, b_offset, 0));
	}

	public static int compare(byte[] a, byte[] b, int b_offset, int length)
	{
		//compare between both byte buffer, return number of bytes that are different
		//'byte a' (shortest length) is usually the bytes to be compare within 'byte b' 
		//byte length to compare is the shortest byte length among the 2 buffer
		//length= number of bytes to compare, if 0 the function will use the shortest length of the 2 array
		int y=0;
		int b_length=b.length-b_offset;

		if(length==0)			//if length is no given
			length=a.length;	//let length be array a size
		else if(a.length<length)
			length=a.length;	//set the lowest byte length of the two byte array, to avoid pointer out of bound
		if(b_length<length)		//ensure length is within both array boundary 
			length=b_length;
		
		for(int x=length-1;x>=0;x--)
		{
			if(a[x]!=b[x+b_offset])
				y++;
		}
		return(y);
	}

	// table to convert a nibble to a hex char.
	static char hex2Char[] = {
		'0' , '1' , '2' , '3' ,
			'4' , '5' , '6' , '7' ,
	 	'8' , '9' , 'A' , 'B' ,
	 	'C' , 'D' , 'E' , 'F'};
}
