package utilities.thread;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * @author Siong Boon
 * @date 08 Dec 2010
 * @comment Abstract Class implementing standard Method & Variables used when Extending the Thread Class
 */

public abstract class UtilitiesThread extends Thread
{
	protected static transient final Log logger = LogFactory.getLog(UtilitiesThread.class);

	//flag to control process flow to be in syn with other critical function
	protected static boolean fIsSynProcess = false;
	//object holding the reference to this thread
	protected Thread fThisThread = null;

	final public void run()			//put final to prevent method from override in the child classes
	{ //getting the thread process ID
		this.fThisThread = Thread.currentThread();
		String threadName = getClass().getName() + " : " + this.fThisThread.getName();
		logger.debug("...thread Started: " + threadName);

		runProcess();
		
		logger.debug("...thread Stopped: " + threadName);
	}
	
	protected abstract void runProcess();	//similar to the run method 
	
	/*
	public void runProcess()		//can be overridden by child class
	{
		//Method 1 to design this run class into 3 callback function (recommended code structure)
  		initProcessCallback();
		while((!isStopRequested()))
		{
			synProcess();			//thread will be blocked here if there is a request to hold the thread.
			runProcessCallback();	//user will implement the codes
		}
		cleanProcessCallback();
	}
	
	protected abstract void initProcessCallback();	//implementing initialise codes before running the runProcess()  
	protected abstract void runProcessCallback();	//implementing codes that will be run within the run() method 
	protected abstract void cleanProcessCallback();	//implementing cleaning codes after terminating the runProcess() 
	*/
	
	final protected synchronized boolean isStopRequested()
	{
		if(fThisThread == null)
			return(true);
		return(false);
	}

	final protected void requestStop()
	{
		Thread tempThread = fThisThread;
		fThisThread = null;
		//interrupt for used to exit from blocking function in the run() method's while loop, is any
		if(tempThread != null)
			tempThread.interrupt();
	}

	final protected void synProcess()
	{
		//hold the thread to prevent further processing if there is a request to hold the thread
		while(isSynProcess()==true)
		{
			try{Thread.sleep(100);}catch(InterruptedException e){}	//delay to prevent loop from wasting computer resources
		}
	}
	
	final private synchronized boolean isSynProcess()
	{
		return(fIsSynProcess);
	}

	final protected synchronized void holdThreadRun() //prevent the run thread to terminate
	{
		fIsSynProcess = true;
	}

	final protected synchronized void releaseThreadRun() //allow the run thread to terminate
	{
		fIsSynProcess = false;
	}
}